//--
//--    Copyright  1994,1995 Uluwatu Software.  All rights reserved.
//--
//--	Author:  Mark Salsbery (CIS 71603,3643)
//--
//-- 	Feel free to use this code any way you wish (at your own risk,
//--	  of course).
//--

#include <owl\owlpch.h>
#include <bwcc.h>
#include <dir.h>
#include <owl\module.h>
#include <owl\dialog.h>
#include <owl\combobox.h>
#include <owl\checkbox.h>
#include <owl\static.h>
#include <owl\opensave.h>
#include <owl\docmanag.h>
#include <owl\docview.h>
#include "testapp.rh"
#include "opsavdlg.h"
#include "docmgr.h"


DEFINE_DOC_TEMPLATE_CLASS(TMyDocument, TWindowView, GenericDocViewTemplate);
GenericDocViewTemplate GDVTplAll("All Files (*.*)",
								 "*.*",
								 0,
								 "",
								 dtAutoDelete | dtUpdateDir | dtAutoOpen |
										 dtHideReadOnly | dtOverwritePrompt);


//------------------------------------------
//  TCustomDocManager::TCustomDocManager()
//------------------------------------------
TCustomDocManager::TCustomDocManager(int mode) : TDocManager(mode)
{
}


//-------------------------------------------
//  TCustomDocManager::~TCustomDocManager()
//-------------------------------------------
TCustomDocManager::~TCustomDocManager()
{
}


//--------------------------------------
//  TCustomDocManager::SelectDocPath()
//--------------------------------------
int TCustomDocManager::SelectDocPath(TDocTemplate** ppTplList,
										int nTplCount,char far *pszPath,
										 int nBufLen,long lFlags,bool fSave)
{
	//
	// Overrides TDocManager's SelectDocPath() to use our own custom
	//   open/save dialog templates
	//

	char szExtBuf[MAXEXT + 1];
	int nIndex, nEndIndex, nCount, nLen, nRetCode;
	char *pszFiltBuf;
	char *pBuf;
	TDocTemplate *pTpl;
	TDVFileSaveDialog *pSaveDlg = 0;
	TDVFileOpenDialog *pOpenDlg = 0;
	OPENFILENAME *pOfn;

	// combine description and filters into a single TOpenSaveDialog
	//   compatible string
	for (nLen = 2,nCount = 0; nCount < nTplCount; ++nCount)
	{
		pTpl = ppTplList[nCount];
		// " + 2" is space for two "|"s
		nLen += (::lstrlen(pTpl->GetFileFilter()) + 2);
		if (pTpl->GetDescription())
			nLen += ::lstrlen(pTpl->GetDescription());
	}
	pszFiltBuf = new char[nLen];
	for (pBuf = pszFiltBuf, nCount=0; nCount < nTplCount; nCount++)
	{
		pTpl = ppTplList[nCount];
		if ((nLen = ::lstrlen(pTpl->GetDescription())) != 0)
		{
			::lstrcpy(pBuf, pTpl->GetDescription());
			pBuf += nLen;
			*pBuf++ = '|';
		}
		else
		{
			*pBuf++ = '|';
		}
		nLen = ::lstrlen(pTpl->GetFileFilter());
		::lstrcpy(pBuf, pTpl->GetFileFilter());
		pBuf += nLen;
		*pBuf++ = '|';
	}
	*pBuf = 0;  // null terminator

	//set selection to previously selected template if present
	for (nIndex = nCount = 0; nCount < nTplCount; ++nCount)
	{
		if (ppTplList[nCount]->IsFlagSet(dtSelected))
		{
			nIndex = nCount;
			break;
		}
	}

	// create & initialize a TFileOpenDialog::TData object
	TFileOpenDialog::TData FNData(0,pszFiltBuf,0,
							  (char *)ppTplList[nIndex]->GetDirectory(),
														   (LPSTR)&szExtBuf);
	FNData.FilterIndex = nIndex;
	// Init Flags to only the Windows "OFN_..." flags
	// Clear the OFN_HIDEREADONLY flag to fool Windows into thinking
	//   it's always there so we can show/hide it whenever we want
	FNData.Flags = (ppTplList[nIndex]->GetFlags() | lFlags) &
												  DTOPENSAVEFLAGSMASK
														 & ~OFN_HIDEREADONLY;
	::lstrcpy(FNData.FileName,pszPath);

	// allocate and execute a "Save As" or "Open" dialog
	if (fSave)
	{
		pSaveDlg = new TDVFileSaveDialog(lFlags,ppTplList,
								  GetApplication()->GetMainWindow(),
														  FNData,"Save As");
		nRetCode = pSaveDlg->Execute();
	}
	else
	{
		pOpenDlg = new TDVFileOpenDialog(lFlags,ppTplList,
								  GetApplication()->GetMainWindow(),
								                            FNData,"Open");
		nRetCode = pOpenDlg->Execute();
	}

	if (nRetCode == IDOK)
	{
		// copy user-entered pathname back to caller's buffer
		if (::lstrlen(FNData.FileName) < nBufLen)
		{
			::lstrcpy(pszPath,FNData.FileName);
		}
		else
		{
			pszPath[0] = 0;
			TRACEX(UluDocMan,0,"TCustomDocManager::SelectDocPath()"
									" - Caller's Pathname Buffer Too Small");
		}

		// get pointer to OPENFILENAME struct
		if (fSave)  pOfn = pSaveDlg->GetOFNPtr();
		else       pOfn = pOpenDlg->GetOFNPtr();

		// nEndIndex = last template index selected (1 based!!!)
		nEndIndex = (int)pOfn->nFilterIndex;

		// set flag to remember which template was selected
		for (nIndex = nEndIndex - 1, nCount = 0;
											 nCount < nTplCount; ++nCount)
		{
			if (nCount == nIndex)  ppTplList[nCount]->SetFlag(dtSelected);
			else                   ppTplList[nCount]->ClearFlag(dtSelected);
		}

		// update template with directory from dialog if flag set
		pTpl = ppTplList[nEndIndex - 1];
//** Borland Bug - Doesn't update directory if directory is root of previous
//**   directory for the template.  This looked like a performance
//**   enhancement attempt - I removed it.  (09/21/94)
		//if (pOfn->nFileOffset && (pTpl->IsFlagSet(dtUpdateDir))
		//				 && (pTpl->GetDirectory() == 0
		//					 || memcmp(pTpl->GetDirectory(),
		//						 pOfn->lpstrFile,pOfn->nFileOffset) != 0))
		//{
		if (pOfn->nFileOffset && (pTpl->IsFlagSet(dtUpdateDir)))
		{
//** Borland Bug - Passes trailing backslash with path to SetDirectory()
//**   so next time around Windows refuses to use the path.  I adjusted it
//**   so the trailing backslash is only left intact if it's the root
//**   directory.  (09/21/94)
			//pTpl->SetDirectory(pOfn->lpstrFile, pOfn->nFileOffset - 1);
			pTpl->SetDirectory(pOfn->lpstrFile, pOfn->nFileOffset -
							   ((pOfn->lpstrFile[pOfn->nFileOffset - 2] !=
															  ':') ? 1 : 0));
		}
	}
	else  // user cancelled or error
	{
		DWORD dwErr = FNData.Error;
		if (dwErr != 0)  TRACEX(UluDocMan,0,"TOpenSaveDialog error "
																   << dwErr);
		nEndIndex = 0;
	}

	// clean up
	delete pSaveDlg;
	delete pOpenDlg;
	delete[] pszFiltBuf;

	return nEndIndex;
}



DEFINE_RESPONSE_TABLE1(TDVFileOpenDialog, TCustomFileOpenDialog)
	EV_WM_PAINT,
	EV_CBN_SELCHANGE(IDC_CB_FILETYPES, FileTypesBoxSelChg),
END_RESPONSE_TABLE;


//--------------------------------------------
//  TDVFileOpenDialog::TDVFileOpenDialog()
//--------------------------------------------
TDVFileOpenDialog::TDVFileOpenDialog(long lOrigFlags,
									 TDocTemplate **ppTmplList,
										 TWindow *pParent,TData &data,
														  LPCSTR pTitle,
															 TModule *pMod) :
									TCustomFileOpenDialog(pParent,data,
																 pTitle,pMod)
{
	// Initialize object
	lOriginalFlags = lOrigFlags;
	ppTplList = ppTmplList;
	ofn.nFilterIndex = Data.FilterIndex + 1;
	ofn.Flags |= OFN_NOCHANGEDIR;

	// create control objects
	pFileTypesCmbBox = new TComboBox(this,IDC_CB_FILETYPES);
	pReadOnlyChkBox = new TCheckBox(this,IDC_BB_READONLY);
}


//---------------------------------------------
//  TDVFileOpenDialog::~TDVFileOpenDialog()
//---------------------------------------------
TDVFileOpenDialog::~TDVFileOpenDialog ()
{
	Destroy();
}


//-------------------------------------
//  TDVFileOpenDialog::SetupWindow()
//-------------------------------------
void TDVFileOpenDialog::SetupWindow()
{
	TCustomFileOpenDialog::SetupWindow();

	nCurTplIndex = Data.FilterIndex;
	Update();
}


//---------------------------------
//  TDVFileOpenDialog::EvPaint()
//---------------------------------
void TDVFileOpenDialog::EvPaint()
{
	// Let OWL do the default painting
	TCustomFileOpenDialog::EvPaint();
	UpdateWindow();

	// now call Update() to update the "Read Only" check box
	Update();
}


//--------------------------------
//  TDVFileOpenDialog::Update()
//--------------------------------
void TDVFileOpenDialog::Update()
{
	long lFlags = ppTplList[nCurTplIndex]->GetFlags() | lOriginalFlags;

	// update OPENFILENAME struct Flags
	ofn.Flags = lFlags & ~dtProhibited;

	// update default extension string to reflect current selected template
	if (ppTplList[nCurTplIndex]->GetDefaultExt())
	{
		::lstrcpy(Data.DefExt, ppTplList[nCurTplIndex]->GetDefaultExt());
	}
	else
	{
		Data.DefExt[0] = 0;
	}

	// update "Read Only" check box
	pReadOnlyChkBox->SetCheck((lFlags & dtReadOnly) ? BF_CHECKED :
															   BF_UNCHECKED);
	pReadOnlyChkBox->Show((lFlags & dtHideReadOnly) ? SW_HIDE : SW_SHOW);
}


//--------------------------------------------
//  TDVFileOpenDialog::FileTypesBoxSelChg()
//--------------------------------------------
void TDVFileOpenDialog::FileTypesBoxSelChg()
{
	nCurTplIndex = pFileTypesCmbBox->GetSelIndex();
	Update();

	DefaultProcessing();
}



DEFINE_RESPONSE_TABLE1(TDVFileSaveDialog, TCustomFileSaveDialog)
	EV_WM_PAINT,
	EV_CBN_SELCHANGE(IDC_CB_FILETYPES, FileTypesBoxSelChg),
END_RESPONSE_TABLE;



//--------------------------------------------
//  TDVFileSaveDialog::TDVFileSaveDialog()
//--------------------------------------------
TDVFileSaveDialog::TDVFileSaveDialog(long lOrigFlags,
									  TDocTemplate **ppTmplList,
										TWindow *pParent,TData &data,
														LPCSTR pTitle,
															 TModule *pMod) :
									TCustomFileSaveDialog(pParent,data,
																 pTitle,pMod)
{
	// Initialize object
	lOriginalFlags = lOrigFlags;
	ppTplList = ppTmplList;
	ofn.nFilterIndex = Data.FilterIndex + 1;
	ofn.Flags |= OFN_NOCHANGEDIR;

	// Create control objects
	pFileTypesCmbBox = new TComboBox(this,IDC_CB_FILETYPES);
	pReadOnlyChkBox = new TCheckBox(this,IDC_BB_READONLY);
}


//---------------------------------------------
//  TDVFileSaveDialog::~TDVFileSaveDialog()
//---------------------------------------------
TDVFileSaveDialog::~TDVFileSaveDialog ()
{
	Destroy();
}


//-------------------------------------
//  TDVFileSaveDialog::SetupWindow()
//-------------------------------------
void TDVFileSaveDialog::SetupWindow()
{
	TCustomFileSaveDialog::SetupWindow();

	nCurTplIndex = Data.FilterIndex;
	Update();
}


//---------------------------------
//  TDVFileSaveDialog::EvPaint()
//---------------------------------
void TDVFileSaveDialog::EvPaint()
{
	// Let OWL do the default painting
	TCustomFileSaveDialog::EvPaint();
	UpdateWindow();

	// now call Update() to update the "Read Only" check box
	Update();
}


//--------------------------------
//  TDVFileSaveDialog::Update()
//--------------------------------
void TDVFileSaveDialog::Update()
{
	long lFlags = ppTplList[nCurTplIndex]->GetFlags() | lOriginalFlags;

	// Save dlg only...
	// don't need Read Only check box and file doesn't have to exist
	lFlags = (lFlags | dtNoReadOnly) & ~dtFileMustExist;

	// update OPENFILENAME struct Flags
	ofn.Flags = lFlags & ~dtProhibited;

	// update default extension string to reflect current selected template
	if (ppTplList[nCurTplIndex]->GetDefaultExt())
	{
		::lstrcpy(Data.DefExt, ppTplList[nCurTplIndex]->GetDefaultExt());
	}
	else
	{
		Data.DefExt[0] = 0;
	}

	// update "Read Only" check box
	pReadOnlyChkBox->SetCheck((lFlags & dtReadOnly) ? BF_CHECKED :
															   BF_UNCHECKED);
	pReadOnlyChkBox->Show((lFlags & dtHideReadOnly) ? SW_HIDE : SW_SHOW);
}


//--------------------------------------------
//  TDVFileSaveDialog::FileTypesBoxSelChg()
//--------------------------------------------
void TDVFileSaveDialog::FileTypesBoxSelChg()
{
	nCurTplIndex = pFileTypesCmbBox->GetSelIndex();
	Update();

	DefaultProcessing();
}


